require 'paths'
require 'pp'

class Configuration

  attr_reader :full_traversal

  def initialize(&block)
    @full_traversal = [:beginning]
    instance_eval(&block)
  end

  def visit_record(fields = {})
    data_entry(:add_visit_page, :case_display_page, fields)
  end

  def audit_record(fields = {})
    needs(:add_visit_page)
    data_entry(:add_audit_page, :case_display_page, fields)
  end

  def case_record(fields = {})
    data_entry(:add_case_page, :case_display_page, fields)
  end
                

  # Note that the caller of this method is assuming that getting to
  # a page implies the creation of needed data along the way.
  def needs(page)
    unless @full_traversal.any? { | t | t.class != Symbol && t.destination == page }
      move_to(page)
    end
  end

  def we_are_at(destination)
    path = Path.between(@full_traversal.last, destination)
    @full_traversal += path.path
  end
  alias_method(:be_at, :we_are_at)
  alias_method(:move_to, :we_are_at)

  def make_app
    app = App.new
    @full_traversal.each { | node |
      if node.respond_to?(:implement_through)
        node.implement_through(app)
      end
    }
    app
  end

  private

  def data_entry(entry_page, post_submit_page, fields)
    be_at(entry_page)
    move_to(post_submit_page)
    fields.each_pair { | field, value | replace_default_with(field, value) }
  end


  # Hackish: replace on most recent transition that has that field.
  # Plus go wild with that ol' functional style. Whee!
  #
  # Kludge: this is terribly unsafe if more than one transition can have
  # a particular field. 
  def replace_default_with(field, value)
    @full_traversal.reverse.reject { | t |
      t.is_a?(Symbol)
    }.find { | t |
      t.has_field?(field)
    }.set_field(field, value)
  end
    
end
