require 'declarative-test-util'

class CaseDisplayPageTests < Test::Unit::TestCase

  def test_typical_case_display_page
    given_app_with {
      case_record('clinic_id' => 19600219)
    }
    when_rendering(:case_display_page) {
      assert_page_title_matches(/^Case 19600219/)
      assert_page_has_action(:want_add_visit_form)
      assert_page_has_action(:want_add_audit_form)
      assert_page_has_popup_link('help?topic=case_display_page')
    }
  end

  def test_nominal_audit_prevents_the_add_audit_action
    given_app_with {
      audit_record('variance' => 'nominal')
    }
    when_rendering(:case_display_page) { 
      assert_page_has_no_action(:want_add_audit_form)
    }
  end


  # These are more unit-ish tests of the App, whereas the above are
  # more business-facing. Were this not a spike, the below should be
  # in a separate file.

  # Note that I'm testing two things here:
  # 1. the business rules that determine whether audit actions are allowed.
  # 2. how those business rules are used by app when handed a template.
  # They might be separated.

  def test_audit_actions_allowed_if_none_exist
    given_app_with {
      case_record
    }
    assert_app_fills_template_with_action('template_for_want_add_audit_form',
                                          :want_add_audit_form)
  end

  def test_can_normally_add_multiple_audits
    given_app_with {
      audit_record('variance' => 'high')
      audit_record('variance' => 'low')
    }
    assert_app_fills_template_with_action('template_for_want_add_audit_form',
                                          :want_add_audit_form)
  end

  def test_nominal_add_audit_action_disallows_further_audits
    given_app_with {
      audit_record('variance' => 'nominal')
    }
    assert_app_ignores_template(:template_for_want_add_audit_form)
  end

end
