require 'html-formatting'
require 'util'

class Renderer
  include HtmlFormatting

  def initialize(app)
    @app = app
  end

  def javascript
    %Q{.<script language="javascript">
       .  function standard_popup(url) {
       .     window.open(url, "std_popup", "resizable=1,height=500,width=600,top=0,left="+(screen.availWidth-600))
       .  }
       .</script>}.unindent
  end

  # Rendering entire pages

  def ping_page
    page("Server Alive")
  end

  def beginning_page
    page('Welcome to the Case Management System',
         p({:align => 'center'},
           "Authorized Users", b("Only")),
         command_form('login',
                      labeled_textfield('login'),
                      labeled_textfield('password'),
                      submit('Login')))
  end


  def main_page
    page("Cases Available to #{@app.user.capitalize}",
         p({ :align => 'center'}, "No cases are available."),
         command_form('want_new_case_form',
                      submit('Add a New Case')))
  end


  def gather_case_input_page
    page("Add a Case",
         command_form('record_case',
                      labeled_textfield('client'),
                      labeled_textfield('clinic_id'),
                      submit('Add')))
  end

  def case_display_page
    case_record = @app.current_record
    page("Case #{case_record.clinic_id}",
         p("Owner: #{case_record.client}"),
         visit_list(case_record.visits),
         audit_list(case_record.audits),
         add_visit_button,
         add_audit_button,
         help_link_for('case_display_page'))
  end


  def gather_visit_input_page
    case_record = @app.current_record
    page("Add a visit to case #{case_record.clinic_id}.",
         command_form('record_visit',
                      labeled_textfield('diagnosis'),
                      labeled_textfield('charges'),
                      submit('Add')))
  end

  def gather_audit_input_page
    case_record = @app.current_record
    page("Add an audit to case #{case_record.clinic_id}.",
         command_form('record_audit',
                      labeled_textfield('auditor'),
                      labeled_textfield('variance'),
                      submit('Add')))
  end



  # Rendering snippets of pages.

  def visit_list(visits)
    unless visits.empty?
      p("Visits: ", br,
        ul(visits.collect { | visit |
             li("#{visit.diagnosis} (#{visit.charges})")
           }))
    end
  end

  def audit_list(audits)
    unless audits.empty?
      p("Audits: ", br,
        ul(audits.collect { | audit |
             li("#{audit.auditor} found #{audit.variance} variance.")
           }))
    end
  end

  def add_audit_button
    @app.fill('template_for_want_add_audit_form') { | action | 
      p(command_form(action,
                     submit('Add an Audit Record')))
    }
  end

  def add_visit_button
    p(command_form('want_add_visit_form',
                   submit('Add a Visit')))
  end

  def help_link_for(topic)
    @app.fill(:template_for_help_link, topic) { | action |
      %Q{<a href="javascript:standard_popup('#{action}?topic=#{topic}')">Help</a>}
    }
  end


end
